package gs

import java.io.File
import scala.util.Try

object Git {

  /** Get the latest tag on this repository and attempt to parse it as a
    * [[SemVer]]. This value is retrieved using the `git` command. If for some
    * reason the call fails, there are no tags, or the tag value cannot be
    * parsed, the default [[SemVer]] (`0.1.0`) is used.
    *
    * This function assumes the current directory is a Git repository. The
    * command executed is: `git describe --tags --abbrev=0`
    *
    * @return
    *   The latest semantic version according to the current Git repository.
    */
  def getLatestSemVer(): SemVer = {
    // Suppress all error output. Suppress exceptions for the failed process.
    // Capture the standard output and parse it if the command succeeds.
    val result = os
      .proc("git", "describe", "--tags", "--abbrev=0")
      .call(
        stderr = os.ProcessOutput(
          (
            _,
            _
          ) => ()
        ),
        check = false
      )

    if (result.exitCode != 0)
      SemVer.DefaultVersion
    else
      SemVer.parse(result.out.text()).getOrElse(SemVer.DefaultVersion)
  }

}
