package gs

import sbt._

object SemVerPlugin extends AutoPlugin {
  override def trigger = allRequirements

  val autoImport = SemVerKeys

  import autoImport._

  // Perform all version calculations and expose as a variable.
  lazy val semVerDefaults: Seq[Setting[_]] = {
    // The latest semantic version relevant to this project, calculated by
    // looking at the latest Git tag.
    val latestSemVer =
      Git.getLatestSemVer()

    // The selected release type (if any), determined by looking at the SBT
    // command line and checking if `-Drelease=<release-type>` was specified.
    val releaseType =
      PluginProperties.getReleaseType()

    // See whether this is a pre-release build or not. If it is (if this value
    // is true), the pre-release suffix `-SNAPSHOT` will be appended to the
    // calculated version.
    val isSnapshot =
      PluginProperties.isSnapshot() || releaseType.isEmpty

    // Calculate the appropriate semantic version for this build of the project.
    val selectedSemVer = releaseType match {
      case Some(ReleaseType.Major) => latestSemVer.incrementMajor()
      case Some(ReleaseType.Minor) => latestSemVer.incrementMinor()
      case Some(ReleaseType.Patch) => latestSemVer.incrementPatch()
      case None                    =>
        // Note: this doesn't play nicely with repositories that have
        // pre-release versions before 0.1.0.
        if (latestSemVer == SemVer.DefaultVersion)
          latestSemVer
        else
          latestSemVer.incrementPatch()
    }

    // Expose the relevant values as setting keys.
    Seq(
      semVerCurrent  := latestSemVer.toString(),
      semVerSelected := selectedSemVer.render(isSnapshot),
      semVerMajor    := selectedSemVer.major
    )
  }

  // Automatically exposed globally.
  override lazy val globalSettings: Seq[Setting[_]] = semVerDefaults

  // Add the custom task.
  override lazy val buildSettings: Seq[Setting[_]] = Seq(
    semVerInfo := {
      val log = Keys.streams.value.log
      log.info(s"[SemVer] Current: ${semVerCurrent.value}")
      log.info(s"[SemVer] Selected: ${semVerSelected.value}")
    }
  )

}
